@@ -2,7 +2,7 @@
 # extension modules created with SIP.  It provides information about file
 # locations, version numbers etc., and provides some classes and functions.
 #
-# Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
+# Copyright (c) 2013 Riverbank Computing Limited <info@riverbankcomputing.com>
 #
 # This file is part of SIP.
 #
@@ -597,6 +597,14 @@ class Makefile:
                         defines.append("QT_XMLPATTERNS_LIB")
                     elif mod == "phonon":
                         defines.append("QT_PHONON_LIB")
+
+                    if qt_version >= 0x050000:
+                        if mod == "QtTest":
+                            defines.append("QT_GUI_LIB")
+
+                        if mod in ("QtSql", "QtTest"):
+                            defines.append("QT_WIDGETS_LIB")
+
             elif self._threaded:
                 defines.append("QT_THREAD_SUPPORT")
 
@@ -716,25 +724,39 @@ class Makefile:
                         if mod == "QAxContainer":
                             incdir.append(os.path.join(qtincdir[0], "ActiveQt"))
                         elif self._is_framework(mod):
+                            idir = libdir_qt[0]
+
                             if mod == "QtAssistant" and qt_version < 0x040202:
                                 mod = "QtAssistantClient"
 
-                            incdir.append(os.path.join(libdir_qt[0],
+                            incdir.append(os.path.join(idir,
                                     mod + ".framework", "Headers"))
 
-                            if qt_version >= 0x050000 and mod == "QtGui":
-                                incdir.append(os.path.join(libdir_qt[0],
-                                        "QtWidgets.framework", "Headers"))
-                                incdir.append(os.path.join(libdir_qt[0],
-                                        "QtPrintSupport.framework", "Headers"))
+                            if qt_version >= 0x050000:
+                                if mod == "QtGui":
+                                    incdir.append(os.path.join(idir,
+                                            "QtWidgets.framework", "Headers"))
+                                    incdir.append(os.path.join(idir,
+                                            "QtPrintSupport.framework",
+                                            "Headers"))
+                                elif mod == "QtWebKit":
+                                    incdir.append(os.path.join(idir,
+                                            "QtWebKitWidgets.framework",
+                                            "Headers"))
                         else:
-                            incdir.append(os.path.join(qtincdir[0], mod))
+                            idir = qtincdir[0]
 
-                            if qt_version >= 0x050000 and mod == "QtGui":
-                                incdir.append(os.path.join(qtincdir[0],
-                                        "QtWidgets"))
-                                incdir.append(os.path.join(qtincdir[0],
-                                        "QtPrintSupport"))
+                            incdir.append(os.path.join(idir, mod))
+
+                            if qt_version >= 0x050000:
+                                if mod == "QtGui":
+                                    incdir.append(os.path.join(idir,
+                                            "QtWidgets"))
+                                    incdir.append(os.path.join(idir,
+                                            "QtPrintSupport"))
+                                elif mod == "QtWebKit":
+                                    incdir.append(os.path.join(idir,
+                                            "QtWebKitWidgets"))
 
                 # This must go after the module include directories.
                 incdir.extend(qtincdir)
@@ -795,11 +817,15 @@ class Makefile:
 
         mname is the name of the module.
         """
+        qt_version = self.config.qt_version
+
         if mname == "QtAssistant":
-            if self.config.qt_version >= 0x040202 and sys.platform == "darwin":
+            if qt_version >= 0x040202 and sys.platform == "darwin":
                 lib = mname
             else:
                 lib = "QtAssistantClient"
+        elif mname == "QtWebKit" and qt_version >= 0x050000:
+            lib = "QtWebKitWidgets"
         else:
             lib = mname
 
@@ -811,7 +837,7 @@ class Makefile:
             elif sys.platform == "darwin":
                 if not self._is_framework(mname):
                     lib = lib + "_debug"
-            elif self.config.qt_version < 0x040200:
+            elif qt_version < 0x040200:
                 lib = lib + "_debug"
 
         if sys.platform == "win32" and "shared" in self.config.qt_winconfig.split():
@@ -820,9 +846,12 @@ class Makefile:
                           "QtScript", "QtScriptTools", "QtSql", "QtSvg",
                           "QtTest", "QtWebKit", "QtXml", "QtXmlPatterns",
                           "phonon") or
-                (self.config.qt_version >= 0x040200 and mname == "QtAssistant")):
+                (qt_version >= 0x040200 and mname == "QtAssistant")):
                 lib = lib + "4"
 
+        if sys.platform.startswith("linux") and qt_version >= 0x050000:
+            lib = "Qt5" + lib[2:]
+
         return lib
 
     def optional_list(self, name):
@@ -906,9 +935,11 @@ class Makefile:
 
         libs = self._extract_value(prl_name, "QMAKE_PRL_LIBS").split()
 
-        if self.config.qt_version >= 0x050000 and clib == "QtGui":
+        if self.config.qt_version >= 0x050000 and clib in ("QtGui", "Qt5Gui"):
             for xtra in ("QtWidgets", "QtPrintSupport"):
-                libs.extend(self.platform_lib(xtra, framework).split())
+                libs.extend(
+                        self.platform_lib(
+                                self._qt4_module_to_lib(xtra), framework).split())
 
         return libs
 
@@ -1482,8 +1513,12 @@ class ModuleMakefile(Makefile):
         self.LFLAGS.extend(self.optional_list(lflags_console))
 
         if sys.platform == "darwin":
-            # 'real_prefix' exists if virtualenv is being used.
-            dl = getattr(sys, 'real_prefix', sys.exec_prefix).split(os.sep)
+            from distutils.sysconfig import get_python_inc
+
+            # The Python include directory seems to be the only one that uses
+            # the real path even when using a virtual environment (eg. pyvenv).
+            # Note that I can't remember why we need a framework build.
+            dl = get_python_inc().split(os.sep)
 
             if "Python.framework" not in dl:
                 error("SIP requires Python to be built as a framework")
@@ -1795,6 +1830,9 @@ class ProgramMakefile(Makefile):
 
         build.extend(self.optional_list("CXXFLAGS"))
 
+        # This is for Qt5.
+        build.extend(self.optional_list("CXXFLAGS_APP"))
+
         # Borland requires all flags to precede all file names.
         if self.generator != "BMAKE":
             build.append(source)
@@ -2144,6 +2182,9 @@ def create_config_module(module, template, content, macros=None):
 
         line = sf.readline()
 
+    df.close()
+    sf.close()
+
 
 def version_to_sip_tag(version, tags, description):
     """Convert a version number to a SIP tag.
@@ -2305,10 +2346,13 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
                 self._openfile(nextfile)
                 return self.readline()
 
-            if not line and self.filestack:
-                self.currentfile = self.filestack.pop()
-                self.path = self.pathstack.pop()
-                return self.readline()
+            if not line:
+                self.currentfile.close()
+
+                if self.filestack:
+                    self.currentfile = self.filestack.pop()
+                    self.path = self.pathstack.pop()
+                    return self.readline()
 
             return line
 
